/*
 * A file accessor employing a rewritable memory area.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <algorithm>

#include "block_file_accessor.h"



BlockFileAccessor::BlockFileAccessor(const char *path, fileid_t fileid)
: Accessor(fileid)
{
    /* Obtain the size of the file. */

    struct stat buf;

    if (stat(path, &buf))
    {
        _size = 0;
        return;
    }

    /* Allocate memory. */

    _size = buf.st_size;
    _data = (char *) malloc(_size);

    if (_data == NULL)
        return;

    /* Load the file into memory and initialise the size. */

    FILE *fp = fopen(path, "r");

    if (fp == NULL)
    {
        free(_data);
        return;
    }

    _size = fread(_data, sizeof(char), _size, fp);

    fclose(fp);
}

/* Update the size of the file. */

void BlockFileAccessor::set_size(offset_t size)
{
    void *new_data = realloc(_data, size);

    if (new_data != NULL)
    {
        _data = (char *) new_data;

        if (size > _size)
            memset(_data + _size, 0, size - _size);

        Accessor::set_size(size);
    }
}

/* Data transfer helper methods. */

/* Fill the populated portion of a flexpage. */

void BlockFileAccessor::fill_populated(Flexpage *flexpage)
{
    offset_t filepos = flexpage->base_offset;
    offset_t addr = flexpage->base_addr;
    offset_t populated_size = std::min(flexpage->size, _size - filepos);

    /* Tag the region with file state. */

    flexpage->region->fill(fileid, filepos);

    /* Fill the region with file content. */

    memcpy((void *) addr, _data + filepos, populated_size);

    /* Pad the flexpage with zero. */

    if (populated_size < flexpage->size)
        memset((void *) (addr + populated_size), 0, flexpage->size - populated_size);
}

/* Flush the populated portion of a flexpage. */

void BlockFileAccessor::flush_populated(Flexpage *flexpage)
{
    offset_t filepos = flexpage->base_offset;
    offset_t addr = flexpage->base_addr;
    offset_t populated_size = std::min(flexpage->size, _size - filepos);

    /* Remove the file state tag from the region. */

    flexpage->region->flush();

    /* Copy the populated region to the block memory. */

    memcpy((void *) (_data + filepos), (const void *) addr, populated_size);
}

// vim: tabstop=4 expandtab shiftwidth=4
